"""Preprocess in-situ observation data."""

# !/usr/bin/env python
# -*- coding: utf-8 -*-

# builtins
import re
import sys
import logging
from logging.config import dictConfig

# externals
import numpy as np
import xarray as xr

# locals
from climax.core.cli import preprocess_obs_parser
from climax.core.constants import CORDEX_VARIABLES, PROJECTION
from pysegcnn.core.logging import log_conf
from pysegcnn.core.utils import search_files
from pysegcnn.core.trainer import LogConfig

# module level logger
LOGGER = logging.getLogger(__name__)


if __name__ == '__main__':

    # initialize logging
    dictConfig(log_conf())

    # define command line argument parser
    parser = preprocess_obs_parser()

    # parse command line arguments
    args = sys.argv[1:]
    if not args:
        parser.print_help()
        sys.exit()
    else:
        args = parser.parse_args(args)

    # check whether the source directory exists
    if args.source.exists():

        # check whether a single variable is specified
        variables = CORDEX_VARIABLES
        if args.variable is not None:
            variables = args.variable

        # iterate over the variables to preprocess
        for var in variables:
            # path to files of the current variable
            source = sorted(search_files(args.source.joinpath(var),
                                         '(.*).nc$'))

            # get observation time period
            years = [re.search('[0-9]{4}', str(f))[0] for f in source]
            y_min, y_max = min(years), max(years)
            LogConfig.init_log('Aggregating files for years: {}'
                               .format('-'.join([y_min, y_max])))
            LOGGER.info(('\n ' + (len(__name__) + 1) * ' ').join(
                        ['{}'.format(file) for file in source]))

            # check for dry-run
            if args.dry_run:
                LOGGER.info('Dry run: No output produced.')
                continue

            # check if aggregated file exists
            filename = '_'.join(['OBS', var, y_min, y_max]) + '.nc'
            filename = args.target.joinpath(var, filename)
            if not filename.parent.exists():
                LOGGER.info('mkdir {}'.format(filename.parent))
                filename.parent.mkdir(parents=True, exist_ok=True)
            if filename.exists() and not args.overwrite:
                LOGGER.info('{} already exists.'.format(filename))
                continue

            # aggregate files for different years into a single file using
            # xarray and dask
            LOGGER.info('Aggregating different years into single file ...')
            ds = xr.open_mfdataset(source, parallel=True).compute()

            # drop projection variable, if present
            ds = ds.drop_vars(PROJECTION) if PROJECTION in ds.data_vars else ds

            # rename temporal dimension: from DATE to time
            ds = ds.rename({'DATE': 'time'})

            # rename data variable
            if 'temperature' in ds.data_vars:
                ds = ds.rename({'temperature': var})

            # center hours at 00:00:00 rather than 12:00:00
            ds['time'] = np.asarray([t.astype('datetime64[D]') for t in
                                     ds.time.values])

            # save aggregated netcdf file
            LOGGER.info('Saving NetCDF: {}'.format(filename))
            ds.to_netcdf(filename, engine='h5netcdf')

    else:
        LOGGER.info('{} does not exist.'.format(str(args.source)))
        sys.exit()
